import 'package:pip_services3_commons/pip_services3_commons.dart';
import 'package:pip_services_accounts/src/data/version1/AccountV1.dart';
import 'package:pip_services3_commons/src/data/PagingParams.dart';
import 'package:pip_services3_commons/src/data/FilterParams.dart';
import 'package:pip_services3_commons/src/data/DataPage.dart';

import '../../pip_clients_accounts.dart';

class AccountsMemoryClientV1 implements IAccountsClientV1 {
  final _maxPageSize = 100;
  var _accounts = <AccountV1>[];

  AccountsMemoryClientV1({List<AccountV1>? items}) {
    _accounts = items ?? [];
  }

  bool _matchString(String? value, String? search) {
    if (value == null && search == null) return true;
    if (value == null || search == null) return false;
    return value.toLowerCase().contains(search);
  }

  bool _matchSearch(AccountV1 item, String search) {
    search = search.toLowerCase();
    if (_matchString(item.name, search)) return true;
    return false;
  }

  bool Function(AccountV1) _composeFilter(FilterParams? filter) {
    filter = filter ?? FilterParams();
    var search = filter.getAsNullableString('search');
    var id = filter.getAsNullableString('id');
    var name = filter.getAsNullableString('name');
    var login = filter.getAsNullableString('login');
    var active = filter.getAsNullableBoolean('active');
    var fromCreateTime = filter.getAsNullableDateTime('from_create_time');
    var toCreateTime = filter.getAsNullableDateTime('to_create_time');
    var deleted = filter.getAsBooleanWithDefault('deleted', false);

    return (item) {
      if (search != null && !_matchSearch(item, search)) {
        return false;
      }
      if (id != null && id != item.id) {
        return false;
      }
      if (name != null && name != item.name) {
        return false;
      }
      if (login != null && login != item.login) {
        return false;
      }
      if (active != null && active != item.active) {
        return false;
      }
      if (fromCreateTime != null &&
          (item.create_time?.microsecondsSinceEpoch ?? 0) >=
              fromCreateTime.microsecondsSinceEpoch) {
        return false;
      }
      if (toCreateTime != null &&
          (item.create_time?.microsecondsSinceEpoch ?? 0) <
              toCreateTime.microsecondsSinceEpoch) {
        return false;
      }
      if (!deleted && item.deleted != null && item.deleted!) {
        return false;
      }
      return true;
    };
  }

  @override
  Future<DataPage<AccountV1>> getAccounts(
      String? correlationId, FilterParams? filter, PagingParams? paging) async {
    var filterCurl = _composeFilter(filter);
    var accounts = _accounts.where(filterCurl).toList();

    // Extract a page
    paging = paging ?? PagingParams();
    var skip = paging.getSkip(-1);
    var take = paging.getTake(_maxPageSize);

    var total = 0;
    if (paging.total) total = accounts.length;

    if (skip > 0) {
      accounts = accounts.skip(skip).toList();
    }
    accounts = accounts.take(take).toList();

    return DataPage(accounts, total);
  }

  @override
  Future<AccountV1?> getAccountById(String? correlationId, String id) async {
    var accounts = _accounts.where((x) {
      return x.id == id;
    }).toList();
    var account = accounts.isNotEmpty ? accounts[0] : null;

    return account;
  }

  @override
  Future<AccountV1?> getAccountByLogin(
      String? correlationId, String login) async {
    var accounts = _accounts.where((x) {
      return x.login == login;
    }).toList();
    var account = accounts.isNotEmpty ? accounts[0] : null;

    return account;
  }

  @override
  Future<AccountV1?> getAccountByIdOrLogin(
      String? correlationId, String idOrLogin) async {
    var accounts = _accounts.where((x) {
      return x.id == idOrLogin || x.login == idOrLogin;
    }).toList();
    var account = accounts.isNotEmpty ? accounts[0] : null;

    return account;
  }

  @override
  Future<AccountV1?> createAccount(
      String? correlationId, AccountV1? account) async {
    if (account == null) {
      return null;
    }

    var oldAccounts = _accounts.where((x) {
      return x.login == account.login;
    }).toList();
    if (oldAccounts.isNotEmpty) {
      throw BadRequestException(correlationId, 'ACCOUNT_ALREADY_EXIST',
          'Account ' + account.login.toString() + ' already exists');
    }

    // clone
    var newAccount = AccountV1();
    newAccount.fromJson(account.toJson());
    newAccount.id = newAccount.id ?? IdGenerator.nextLong();

    _accounts.add(newAccount);

    return newAccount;
  }

  @override
  Future<AccountV1?> updateAccount(
      String? correlationId, AccountV1? account) async {
    var index = _accounts.indexWhere((el) => el.id == account?.id).toInt();

    if (index < 0 || account == null) {
      return null;
    }

    var newAccount = AccountV1();
    newAccount.fromJson(account.toJson());
    _accounts[index] = newAccount;

    return newAccount;
  }

  @override
  Future<AccountV1?> deleteAccountById(String? correlationId, String id) async {
    var index = _accounts.indexWhere((el) => el.id == id);
    var item = _accounts[index];

    if (index < 0) {
      return null;
    }

    item.deleted = true;
    //this._accounts.sublist(index, 1);

    return item;
  }
}
